"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Option_1 = require("./option/Option");
const OptionCode_1 = require("./option/OptionCode");
const OptionString_1 = require("./option/OptionString");
const OptionObject_1 = require("./option/OptionObject");
const OptionConfig_1 = require("./option/OptionConfig");
const OptionArray_1 = require("./option/OptionArray");
class Config {
    constructor(options) {
        if (options)
            this.push(options);
    }
    get options() {
        return this._options ? this._options : this._options = [];
    }
    set options(options) {
        this._options = options;
    }
    push(data) {
        let optionArray = _.isArray(data) ? data : [data];
        let options = optionArray.map(item => item instanceof Option_1.default ? item : this.createOption(item));
        // Если добавляемая опция уже есть, то имеющаяся заменяется на новую.
        const conflictOptions = options.reduce((search, option) => this.has(option) ? search.concat(option) : search, []);
        conflictOptions.forEach(option => this.remove(option));
        this.options = this.options.concat(options);
        return this;
    }
    /**
     * Конструктор опции конфига на основании типа значения.
     * @param data
     */
    createOption(data) {
        switch (Option_1.default.type(data)) {
            case 'code': return new OptionCode_1.default(data);
            case 'string': return new OptionString_1.default(data);
            case 'object': return new OptionObject_1.default(data);
            case 'config': return new OptionConfig_1.default(data);
            case 'array': return new OptionArray_1.default(data);
            default: return new Option_1.default(data);
        }
    }
    find(name) {
        return this.options.find(option => option.name == name);
    }
    has(name) {
        return !!this.find(name);
    }
    remove(removableOption) {
        _.remove(this.options, option => option.name == removableOption.name);
        return this;
    }
    toJavaScript() {
        const optionDivider = `,${Config.EOL}`;
        const result = this.options.filter(option => !option.isSkip).map(option => option.toJavaScript()).join(optionDivider);
        return `{${result}}`;
    }
    clone() {
        return new Config(this.options.map(o => o.clone()));
    }
}
Config.EOL = '\n';
exports.default = Config;
