"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Config_1 = require("../Config");
const Formatter_1 = require("../../Formatter");
const Comment_1 = require("../Comment");
class Option {
    constructor(data) {
        this.comment = new Comment_1.default;
        this.name = data.name;
        if ('value' in data)
            this.value = data.value;
        if ('code' in data)
            this.code = data.code;
        if ('comment' in data)
            this.comment = data.comment instanceof Comment_1.default ? data.comment : new Comment_1.default(data.comment);
        this.skipIfEmpty = !!data.skipIfEmpty;
    }
    static valueToJavaScript(data) {
        return `${data.value && _.isFunction(data.value.toString) ? data.value.toString() : data.value}`;
    }
    static type(data) {
        return 'code' in data ? 'code' : this.valueType(data.value);
    }
    static valueType(dataValue) {
        if (_.isString(dataValue))
            return 'string';
        else if (_.isPlainObject(dataValue))
            return 'object';
        else if (dataValue instanceof Config_1.default)
            return 'config';
        else if (_.isArray(dataValue))
            return 'array';
        else
            return 'undefined';
    }
    get isSkip() {
        return _.isUndefined(this.value) && _.isUndefined(this.code) ? true : (this.skipIfEmpty ? this.isEmpty : false);
    }
    get isEmpty() {
        return false;
    }
    toJavaScript() {
        const result = [];
        if (!this.comment.isEmpty)
            result.push(this.comment.toJavaScript());
        result.push(`${this.name}: ${this.valueToJavaScript()}`);
        return result.join(Formatter_1.default.EOL);
    }
    valueToJavaScript() {
        return Option.valueToJavaScript(this.value);
    }
    clone() {
        return new Option({
            name: this.name,
            value: _.cloneDeepWith(this.value, obj => _.isFunction(obj.clone) ? obj.clone() : obj),
            code: this.code,
            comment: this.comment,
            skipIfEmpty: this.skipIfEmpty
        });
    }
    clear() { }
}
exports.default = Option;
