"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Option_1 = require("./Option");
const OptionConfig_1 = require("./OptionConfig");
const OptionString_1 = require("./OptionString");
const OptionObject_1 = require("./OptionObject");
class OptionArray extends Option_1.default {
    static valueToJavaScript(dataValue) {
        switch (Option_1.default.valueType(dataValue)) {
            case 'string': return OptionString_1.default.valueToJavaScript(dataValue);
            case 'config': return OptionConfig_1.default.valueToJavaScript(dataValue);
            case 'object': return OptionObject_1.default.valueToJavaScript(dataValue);
            case 'array': return `[${dataValue.map(dataValue => OptionArray.valueToJavaScript(dataValue)).join(', ')}]`;
            default: return Option_1.default.valueToJavaScript(dataValue);
        }
    }
    get isEmpty() {
        return !this.value.length;
    }
    constructor(data) {
        super(data);
        this.uniq = data.uniq || (data => data);
        this.value = this.uniq(this.value || []);
    }
    valueToJavaScript() {
        return OptionArray.valueToJavaScript(this.value);
    }
    push(data) {
        const dataArray = _.isArray(data) ? data : [data];
        this.value = this.value.concat(dataArray);
        this.value = this.uniq(this.value);
        return this;
    }
    clear() {
        this.value = [];
    }
}
exports.default = OptionArray;
