"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Option_1 = require("./Option");
class Config {
    constructor(options) {
        if (options)
            this.push(options);
    }
    get options() {
        return this._options ? this._options : this._options = [];
    }
    set options(options) {
        this._options = options;
    }
    push(data) {
        let optionArray = _.isArray(data) ? data : [data];
        let options = optionArray.map(item => item instanceof Option_1.default ? item : this.createOption(item));
        // Если добавляемая опция уже есть, то имеющаяся заменяется на новую.
        const conflictOptions = options.reduce((search, option) => this.has(option) ? search.concat(option) : search, []);
        conflictOptions.forEach(option => this.remove(option));
        this.options = this.options.concat(options);
        return this;
    }
    createOption(data) {
        switch (Option_1.default.type(data.value)) {
            default: return new Option_1.default(data);
        }
    }
    find(name) {
        return this.options.find(option => option.name == name);
    }
    has(name) {
        return !!this.find(name);
    }
    remove(removableOption) {
        _.remove(this.options, option => option.name == removableOption.name);
        return this;
    }
    toJavaScript() {
        const optionDivider = `,${Config.EOL}`;
        const result = this.options.filter(option => !option.isSkip).map(option => option.toJavaScript()).join(optionDivider);
        return `{${result}}`;
    }
}
Config.EOL = '\n';
exports.default = Config;
